/*

HXED, Version 2.00, a Hexadecimal Editor for Binary Files
Copyright 1991 Anthony F. Stuart, All rights reserved.

Date    Name	Change
910209	AFS	Initial release

*/


#include "curses/screen/curses.h"

#include "sd.h"
#include "def.h"
#include "ext.h"

#define CTRL_A	0x01
#define CTRL_B	0x02
#define CTRL_D	0x04
#define CTRL_F	0x06
#define CTRL_L	0x0c
#define CTRL_U	0x15
#define DEL	0x7f
#define NOP	0

int		 def_attr = A_NORMAL;
static int	 input_mode = FALSE;
static int	 repeat_count = 0;
static int	 repeat_char = 0;

init_hardware ()
{
	initscr ();			/* initialize curses package	*/
	cbreak ();			/* get one key at a time	*/
	noecho ();			/* dont echo keys as entered	*/
	nonl ();			/* dont map newline to cr nl	*/
	intrflush (stdscr, FALSE);	/* flush input on interrupt	*/
	keypad (stdscr, TRUE);		/* enable cursor keys		*/
	idlok (stdscr, TRUE);		/* enable insert/delete line	*/
	scrollok (stdscr, TRUE);	/* enable scroll		*/
	leaveok (stdscr, FALSE);	/* leave cursor at right loc	*/

	header_attr = A_STANDOUT;	/* normally black on white	*/
	text_attr = A_NORMAL;		/* normally white on black	*/
	cursor_attr = A_STANDOUT;	/* normally black on white	*/
	footer_attr = A_STANDOUT;	/* normally black on white	*/
	message_attr = A_NORMAL;	/* normally white on black	*/

	message_row = 22;		/* move msg row up for 80x24	*/
	help_row = 2;			/* move help row up for 80x24	*/
	help_col = 9;			/* move help to right a little	*/
	def_disp = '.';			/* default display character	*/
}

restore_hardware ()
{
	refresh ();			/* display whatevers on screen	*/
	endwin ();			/* terminate curses/restore tty	*/
}

scroll_screen_down ()
{
	/* Unfortunately some versions of curses do not support */
	/* setscrreg(row, row) and scrl(+/-rows) and so this uses */
	/* the ol' ugly delete and insert line trick to scroll. */

	move (text_maxrow, text_col);
	deleteln ();
	move (text_row, text_col);
	insertln ();
}

scroll_screen_up ()
{
	/* Unfortunately some versions of curses do not support */
	/* setscrreg(row, row) and scrl(+/-rows) and so this uses */
	/* the ol' ugly delete and insert line trick to scroll. */

	move (text_row, text_col);
	deleteln ();
	move (text_maxrow, text_col);
	insertln ();
}

clear_screen (attr)
int	 attr;
{
	clear ();
}

truncate_row (row, col, attr)
int	 row;
int	 col;
int	 attr;
{
	register int	 i;

	attron (attr);

	while (col < 80) {
		mvaddch (row, col++, ' ');
	}

	attroff (attr);
}

position_cursor (row, col)
int	 row;
int	 col;
{
	move (row, col);
}

display_string (row, col, buf, attr)
int	 row;
int	 col;
char	*buf;
int	 attr;
{
	attron (attr);

	while (*buf) {
		mvaddch (row, col++, *buf++);
	}

	attroff (attr);
}

display_char (row, col, c, attr)
int	 row;
int	 col;
int	 c;
int	 attr;
{
	attron (attr);
	mvaddch (row, col, c);
	attroff (attr);
}

get_keystroke (type)
int	 type;
{
	int		 c;
	static int	 first_call = TRUE;

	if (repeat_count > 0) {
		repeat_count--;
		return (repeat_char);
	}

	if (first_call) {
		first_call = FALSE;
		print_current_mode ();
	}

	refresh ();

	if (type == DATA && input_mode == FALSE)
		return (getch ());

	for (;;) {
		if (input_mode) {
			c = getch ();
			if (c == CTRL_A) {
				input_mode = FALSE;
				print_current_mode ();
			}
			else {
				return (c);
			}
		}
		else {
			if (c = translate_key (getch ()))
				return (c);
		}
	}
}

translate_key (c)
int	 c;
{
	int	 number;

	number = 0;

	while (c >= '0' && c <= '9') {
		number = (number * 10) + (c - '0');
		c = getch ();
	}

	switch (c) {
		case 'h':
		case KEY_LEFT:
			c = LEFT;
			repeat_count = number-1;
			break;
		case 'k':
		case KEY_UP:
			c = UP;
			repeat_count = number-1;
			break;
		case 'l':
		case KEY_RIGHT:
			c = RIGHT;
			repeat_count = number-1;
			break;
		case 'j':
		case KEY_DOWN:
			c = DOWN;
			repeat_count = number-1;
			break;
		case CTRL_U:
		case CTRL_B:
		case KEY_PPAGE:
			c = PGUP;
			repeat_count = number-1;
			break;
		case CTRL_D:
		case CTRL_F:
		case KEY_NPAGE:
			c = PGDOWN;
			repeat_count = number-1;
			break;
		case CTRL_L:
			clearok (curscr);
			refresh ();
			c = NOP;
			break;
		case DEL:
		case 'd':
			c = DELETE;
			repeat_count = number-1;
			break;
		case 'i':
			input_mode = TRUE;
			print_current_mode ();
			c = NOP;
			break;
		case 'g':
			c = GOTO;
			break;
		case 'G':
			if (number == 0)
				c = END;
			else if (number == 1)
				c = HOME;
			else
				c = GOTO;
			break;
		case 'o':
			c = INSERT;
			break;
		case '/':
		case 's':
			c = SEARCH;
			break;
		case 'r':
			c = REPLACE;
			break;
		case '!':
		case 'u':
			c = SUSPEND;
			break;
		case 'w':
			c = WRITE;
			break;
		case 'x':
			c = HEX;
			break;
		case 'q':
			c = QUIT;
			break;
		default:
			c = HELP;
			beep ();
			break;
	}

	c <<= 8;
	repeat_char = c;
	return (c);
}

print_current_mode ()
{
	int		 row;
	int		 col;

	getyx (stdscr, row, col);

	if (input_mode)
		mvaddstr (1, 20, "Input Mode - Enter Ctrl-A For Edit Mode");
	else
		mvaddstr (1, 20, "Edit Mode - Enter i For Input Mode     ");

	move (row, col);
	refresh ();
}

set_direct_video (state)
int	 state;
{
}

get_attr (attr)
char	*attr;
{
}
